/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.fleet.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Wytse Visser
 */
public abstract class Machine implements Serializable, Lookup.Provider {

    private String name;
    private String status;
    private String year;
    private String type;
    private double longitude, latitude;
    private transient Lookup lookup;
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    public Machine() {
        this(null);
    }

    public Machine(String name) {
        this.name = name;
        // Some default values for development purposes
        longitude = 3.872737635239056;
        latitude = 51.53760475933433;
    }

    @Override
    public Lookup getLookup() {
        if (lookup == null) {
            lookup = Lookups.fixed(
                    new MachineGeographical(this));
        }
        return lookup;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        String oldName = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, oldName, name);
    }
    public static final String PROP_NAME = "name";

    public String getStatus() {
        if (status == null) {
            return "Available";
        }
        return status;
    }

    public void setStatus(String status) {
        String oldStatus = this.status;
        this.status = status;
        getPropertyChangeSupport().firePropertyChange(PROP_STATUS, oldStatus, status);
    }
    public static final String PROP_STATUS = "status";

    public String getType() {
        return type;
    }

    public void setType(String type) {
        String oldType = this.type;
        this.type = type;
        getPropertyChangeSupport().firePropertyChange(PROP_TYPE, oldType, type);
    }
    public static final String PROP_TYPE = "type";

    public String getYear() {
        return year;
    }

    public void setYear(String year) {
        String oldYear = this.year;
        this.year = year;
        getPropertyChangeSupport().firePropertyChange(PROP_YEAR, oldYear, year);
    }
    public static final String PROP_YEAR = "year";

    public Double getLongitude() {
        return longitude;
    }

    public void setLongitude(Double longitude) {
        double oldLongitude = this.longitude;
        this.longitude = longitude;
        getPropertyChangeSupport().firePropertyChange(PROP_LONGITUDE, oldLongitude, longitude);
    }
    public static final String PROP_LONGITUDE = "longitude";

    public Double getLatitude() {
        return latitude;
    }

    public void setLatitude(Double latitude) {
        double oldLatitude = this.latitude;
        this.latitude = latitude;
        getPropertyChangeSupport().firePropertyChange(PROP_LATITUDE, oldLatitude, latitude);
    }
    public static final String PROP_LATITUDE = "latitude";

    protected PropertyChangeSupport getPropertyChangeSupport(){
        if (propertyChangeSupport == null){
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }
    
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    public Sheet createSheet() {
        Sheet sheet = Sheet.createDefault();
        Sheet.Set properties = sheet.get(Sheet.PROPERTIES);
        try {
            properties.put(new MachineProperty(String.class, PROP_NAME, "Name"));
            properties.put(new MachineProperty(String.class, PROP_YEAR, "Year"));
            properties.put(new MachineProperty(String.class, PROP_TYPE, "Type"));
            properties.put(new MachineProperty(String.class, PROP_STATUS, "Status"));
            properties.put(new MachineProperty(Double.class, PROP_LONGITUDE, "Longitude"));
            properties.put(new MachineProperty(Double.class, PROP_LATITUDE, "Latitude"));
        } catch (NoSuchMethodException ex) {
            Exceptions.printStackTrace(ex);
        }
        return sheet;
    }

    protected class MachineProperty extends PropertySupport.Reflection {

        public MachineProperty(Class valueType, String property, String displayName) throws NoSuchMethodException {
            super(Machine.this, valueType, property);
            setDisplayName(displayName);
        }
    }
}